> module Config (readConfig, writeConfig) where

> import Maybe
> import System.Exit
> import Text.XML.HaXml.Parse
> import Text.XML.HaXml.Pretty
> import Text.XML.HaXml.Types

> import Feed
> import Util

> configFile :: IO String
> configFile = do
>   dir <- mydir
>   return (dir ++ "/config")

> contentToGuid :: Content -> String
> contentToGuid (CElem (Elem "guid" _ c)) = foldl (++) [] (map (show . content) c)

> contentToFeed :: Content -> Maybe (Feed, [String])
> contentToFeed (CElem (Elem "feed" _ content)) = do
>   url <- getContent content "url"
>   interval <- getContent content "interval"
>   feed <- case reads interval of
>     (n,_):_ -> newFeed url n
>     []      -> Nothing
>   Just (feed, map contentToGuid (filter (elemMatching "guid") content))
> contentToFeed _ = Nothing

> parseConfig :: Document -> [(Feed, [String])]
> parseConfig (Document _ _ (Elem nm _ content) _) = if nm == progName
>   then mapMaybe contentToFeed content
>   else []

> readConfig :: IO [(Feed, [String])]
> readConfig = do
>   name <- configFile
>   conf <- catch (readFile name) (\ e -> return [])
>   return $ either (\ err -> [])
>                   (\ doc -> parseConfig doc)
>                   (xmlParse' name conf)

> guidToXml :: String -> String
> guidToXml gd = "<guid>" ++ gd ++ "</guid>\n"

> feedToXml :: (Feed, [String]) -> String
> feedToXml (fd, gds) = open ++ url ++ int ++ guids ++ close
>   where open  = "<feed>\n"
>         close = "</feed>\n"
>         url   = "<url>" ++ show (uri fd) ++ "</url>\n"
>         int   = "<interval>" ++ show (interval fd) ++ "</interval>\n"
>         guids = foldl (++) [] $ map guidToXml gds

> createConfig :: [Feed] -> [[String]] -> String
> createConfig fs gds = prolog ++ open ++ feeds ++ close
>   where prolog = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n"
>         open   = "<" ++ progName ++ " version=\"1\">\n"
>         close  = "</" ++ progName ++ ">\n"
>         feeds  = foldl (++) [] $ map feedToXml (zip fs gds)

> writeConfig :: [Feed] -> [[String]] -> IO ()
> writeConfig fs gds = do
>   name <- configFile
>   catch (writeFile name (createConfig fs gds)) (\ e -> return ())

vim:set sw=2 ts=2 et:
