> module Util where

> import Data.Char
> import List
> import System.Directory
> import System.Environment
> import Text.XML.HaXml.Parse
> import Text.XML.HaXml.Pretty
> import Text.XML.HaXml.Types

All this does really is turn on the HTTP file logging, to put all of the HTTP
requests/responses in a file.

> debug :: Bool
> debug = False

When running a haskell program as a 'scripting' language (i.e. using
runhaskell), getProgName returns 'Harsh.lhs', which I can't stand.

> progName :: String
> progName = "harsh"

I prefer to be able to specify where configuration directories are placed based
on environment variables.

> mydir :: IO String
> mydir = do
>   prog <- return progName
>   path <- catch
>             (getEnv ((map toUpper prog) ++ "DIR"))
>             (\e -> getEnv "HOME" >>= (\ h -> return $ h ++ "/." ++ (map toLower prog)))
>   createDirectoryIfMissing True path
>   return path

A helper function for finding the element you want.

> elemMatching str c = case c of
>   CElem (Elem s _ _) -> s == str
>   otherwise          -> False

Another helper function. Given a parsed XML document's Content, we want to find
the data for some element in that content. (For example, in a feed, find its
title.)

> getContent :: [Content] -> String -> Maybe String
> getContent con str = (find (elemMatching str) con) >>= getData
>   where getData (CElem (Elem _ _ c)) = return $ foldl (++) [] (map (show . content) c)

vim:set sw=2 ts=2 et:
